
require 'vcs'
require 'db'
require 'dblog'

require 'revision'
require 'target'
require 'config'
require 'visitor'
require 'session'
require 'builder'
require 'ilogger'
require 'options'
require 'myshell'


class SisyphusClient 
  def initialize(host, config_manager, profile_name, interval, roots, store, log)
    @host = host
    @config_manager = config_manager
    @profile_name = profile_name
    @interval = interval
    @roots = roots
    @store = store
    @log = log
    @repo_factory = RepositoryFactory.new
    @builder = Builder.new(@store, @log)
  end

  def run
    loop do
      do_iteration
    end
  end

  def do_iteration
    @store.connect
    begin 
      build_at(Time.now)
    rescue ShellFailure => e
      line = e.to_s
      add_log_line(line)
      @log.error(line)
    ensure
      @store.disconnect
    end
    wait_interval
  end

  def add_log_line(string)
    dbhost = @store.db_host(@host)
    log = SiLog.new(:line => string, :si_host => dbhost)
    log.save
  end

  def build_at(time)
    visitor(time).build_roots
  end

  def wait_interval
    @log.info("sleeping #{@interval} seconds.")
    sleep(@interval)
  end

  def visitor(time)
    config = @config_manager.configuration(@profile_name, time)
    revision_factory = RevisionFactory.new(config.sources.properties['repositories'], 
                                           config.sources.properties['externals'],
                                           @repo_factory, config.user, config.build_dir, time)
    
    session = Session.new(time, @host)
    target_factory = TargetFactory.new(session, config)
    visitor = Visitor.new(target_factory, revision_factory, @roots, @builder, @log)
    return visitor
  end
  
end


if __FILE__ == $0 then
  options = SisyphusClientCommandlineParser.parse(ARGV)
  host = `uname -a`.chomp
  config_manager = ConfigurationManager.new(options.config_url, options.user, options.password)
  profile_name = options.profile_name
  interval = options.interval
  roots = options.roots
  store = DBStore.new(options.dbconf)
  log = IndentedLogger.new(STDERR)
  if options.verbose then
    log.level = Logger::INFO
  else
    log.level = Logger::WARN
  end
  client = SisyphusClient.new(host, config_manager, profile_name, interval, roots, store, log)
  client.run
end


